/**************************************************************************
 *                                                                        *
 *   Copyright (c) Digital Equipment  Corporation,  1991.   All  Rights   *
 *   Reserved.  Unpublished rights reserved under the copyright laws of   *
 *   the United States.                                                   *
 *                                                                        *
 *   The software  contained  on  this  media  is  proprietary  to  and   *
 *   embodies   the   confidential   technology  of  Digital  Equipment   *
 *   Corporation.  Possession, use, duplication or dissemination of the   *
 *   software  and media is authorized only pursuant to a valid written   *
 *   license from Digital Equipment Corporation.                          *
 *                                                                        *
 *   RESTRICTED RIGHTS LEGEND Use, duplication, or  disclosure  by  the   *
 *   U.S.   Government  is  subject  to  restrictions  as  set forth in   *
 *   Subparagraph  (c)(1)(ii)  of  DFARS  252.227-7013,   or   in   FAR   *
 *   52.227-19, as applicable.                                            *
 *                                                                        *
 **************************************************************************/
#ifdef SCCS
static char sccsid[]="@(#)read_scans.c	5.2 Stardent 91/02/05";
#endif
/*
			Copyright (c) 1989 by
			Stardent Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stardent Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under sccs control at Stardent in:
	/sccs3.p/avs/examples/s.read_scans.c
	
*/
#include <stdio.h>
  
#include <avs/flow.h>
#include <avs/field.h>
  
/******************************************************************************/

/*
 * This is a C example to read in a volume data set.  The specific format
 * is described below. To test this program, there is a script in 
 * /usr/avs/examples called make_scans.scr which creates (in /tmp) the
 * exact files which this module wants to read. To create the scans, run
 * avs -cli and type in: script -play make_scans.scr. CAUTION! This will
 * dump about 1.01 MBytes of data into /tmp!
 */

/* 
 * The function AVSinit_modules is called from the main() routine supplied
 * by AVS.  In it, we call AVSmodule_from_desc with the name of our 
 * description routine.
 */

AVSinit_modules()
{
    int read_scans_desc();

    AVSmodule_from_desc(read_scans_desc);
}

/* The routine "read_scans_desc" is the description routine.  */

read_scans_desc()
{
    int read_scans_compute();  /* declare the compute function (below) */
    
    /* Set the module name and type */
    AVSset_module_name("read scans", MODULE_DATA);

    /* Create an output port for the result */
    AVScreate_output_port("Output Field", 
			  "field 3D scalar int");
    
    /* Tell avs what subroutine to call to do the compute */
    AVSset_compute_proc(read_scans_compute);
}

/* The function "read_scans_compute" reads a volume data set stored as a series
 * of x-y scans contained in sequentially numbered files, e.g.:
 *      scan.000
 *      scan.001
 *      ...
 * Each file is organized in the following format:
 *      header data  (optional)
 *      scan data    (with x varying fastest)
 * The arguments are: the new output field (doubly indirected).
 */

/*
 * The macros below are used to define the data types and dimensions.
 */

#define HEADER_SIZE	705		/* Size, in bytes, of header */
#define HEADER_TYPE	unsigned char	/* type of data in header    */
#define SCANDATA_TYPE	int      	/* type of data in scans     */
#define X_SIZE		64		/* X dimension               */
#define Y_SIZE		64		/* Y dimension               */
#define Z_SIZE		64		/* Z dimension               */

read_scans_compute(output)
    AVSfield_int	**output;
{
    AVSfield_int	*tmp_field;
    FILE		*pf;
    int			dims[3], i, j, nx, ny, nz;
    char		filename[512];
    HEADER_TYPE		header_buf[HEADER_SIZE];
    SCANDATA_TYPE	scan_buf[X_SIZE * Y_SIZE];
    
    /* Free old memory */
    if (*output) 
      free(*output);
    
    /* Set the data dimensions - in this case, we always have a 128x128x128 volume */
    nx = dims[0] = X_SIZE;
    ny = dims[1] = Y_SIZE;
    nz = dims[2] = Z_SIZE;
    
    /* Allocate space for volume data */
    tmp_field = (AVSfield_int *)AVSdata_alloc("field 3D scalar int", dims);

    /* Read each planar scan, and add it into the volume */
    for(i = 0; i < nz; i++) {

	/* Create the filename for this scan (in format scan.000, scan.001, etc) */
	sprintf(filename, "/tmp/scan.%03d.fld", i);

	/* Open the file */
	if ((pf = fopen(filename, "r")) == NULL) {
	    fprintf(stderr, "error on scan %d: cannot read %s\n", i, filename);

	    /* Return 0 to indicate an error occured */
	    return(0);
	}
	
	/* Read the header information for this scan */
	if (HEADER_SIZE)
	  fread(header_buf, sizeof(HEADER_TYPE), HEADER_SIZE, pf);

	/* Read the data for this scan */
	fread(scan_buf, sizeof(SCANDATA_TYPE), nx*ny, pf);

	/* Stuff the data into the AVS field structure */
	for(j = 0; j < nx * ny; j++)
	  tmp_field->data[i*nx*ny + j] = (int)scan_buf[j];
	
	/* Close the file */
	fclose(pf);

	/* Status to stdout */
	printf("read_scans_compute(): finished with scan %d (%s)\n", i, filename);
    }

    /* Set the output pointer to our new data */
    *output = tmp_field;

    /* When we're done, we return 1 to indicate success */
    return(1);
}
